import aiohttp

import config
from .Speakers import ModelDescription


class TTSServer:
    def __init__(self):
        self.session = aiohttp.ClientSession(base_url=config.BASE_URL)

    async def synthesize_text(self, text: str, speaker: ModelDescription) -> bytes:
        async with self.session.post(url=f'/synth/{speaker.engine}/model/{speaker.name}', json={'text': text}) as req:
            req.raise_for_status()
            return await req.content.read()

    async def discovery(self) -> list[ModelDescription]:
        async with self.session.get('/discovery') as req:
            req.raise_for_status()
            return [ModelDescription(**item) for item in await req.json()]
